package com.chinacreator.process.service;

import com.chinacreator.process.dao.QueryPrmDataDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.SQLException;
import java.util.List;

@Service
@Transactional
public class QueryPrmDataService {

    @Autowired
    private QueryPrmDataDao queryPrmDataDao;

    public Integer getDayNub(String spid) throws SQLException {
        return queryPrmDataDao.getDayNub(spid);
    }

    public void insertPrm(Integer nub, String time, String type, String status,String spid) throws SQLException {
        queryPrmDataDao.insertPrm(nub, time, type, status,spid);
    }

    public Integer geMonthNub(String spid) throws SQLException {
        return queryPrmDataDao.getMonthNub(spid);
    }

    public Integer geTotal(String spid) throws SQLException {
        return queryPrmDataDao.geTotal(spid);
    }

    public Integer getDayCancel(String spid) throws SQLException {
        return queryPrmDataDao.getDayCancel(spid);
    }

    public Integer getLastMonthNub(String spid) throws SQLException {
        return queryPrmDataDao.getLastMonthNub(spid);
    }

    public Integer getLastMonthCancel(String spid) throws SQLException {
        return queryPrmDataDao.getLastMonthCancel(spid);
    }

    public String getParName() throws SQLException {
        return queryPrmDataDao.getParName();
    }

    public Integer getOrderNubDay(String prmSpid, String parName) throws SQLException {
        return queryPrmDataDao.getOrderNubDay(prmSpid,parName);
    }

    public Integer getorderNubMonth(String prmSpid, String parName) throws SQLException {
        return queryPrmDataDao.getorderNubMonth(prmSpid,parName);
    }

    public List<String> getParNameList() throws SQLException {
        return queryPrmDataDao.getParNameList();
    }

    public Integer getOrderTotal(String spid, String parName) throws SQLException {
        return queryPrmDataDao.getOrderTotal(spid,parName);
    }

    public void insertOrderPrm(Integer nub, String time, String type, String status,String spid) throws SQLException {
        queryPrmDataDao.insertOrderPrm(nub,time,type,status,spid);
    }

    public Integer getmonthMoney(String spid) throws SQLException {
        return queryPrmDataDao.getmonthMoney(spid);
    }
}
